/*
 * MCP3903 Library
 * Kerry D. Wong
 * http://www.kerrywong.com
 * 5/2014
 * extended by Jennifer Sirtl 7.2016
 */

#include "MCP3903.h"
#include <SPI.h>

MCP3903::MCP3903()
{
    pinMode(pinMOSI, OUTPUT);
    pinMode(pinMISO, INPUT);
    pinMode(pinSPIClock, OUTPUT);
    pinMode(pinCS, OUTPUT);

    digitalWrite(pinCS, HIGH);
	for(byte n=0;n==5;n++) Gain[n]=1;//default after Reset
	ScaleADCValue=true; //Output of Adc is scaled with 1/Gain;
}


//set to 24 bit mode
void MCP3903::reset()
{
    reset(OSR_256);
}

//set to resolution specified by OSR
//OSR_32  11 = 256
//OSR_64  10 = 128
//OSR_128 01 = 64 (POR default)
//OSR_256 00 = 32
void MCP3903::reset(byte osr)
{
	unsigned long cmd1 = 0xfc0fd0;//Reset Channels, Dither on, osr=64, AMCLK=MCLK,int.Ref, Cystal
	unsigned long cmd2 = 0x000fc0 | osr << 4;//Dither on, osr=OSR_x, AMCLK=MCLK,int.Ref, Cystal
	byte cmdByte = 	DEVICE_ADDR | REG_CONFIG << 1;

	writeRegister(REG_CONFIG, cmd1);
	writeRegister(REG_CONFIG, cmd2);
}

//read from specified register
//returns 24 bit data
unsigned long MCP3903::readRegister(byte reg)
{
	byte cmdByte = DEVICE_ADDR | reg <<1 | 1;
	unsigned long r = 0;

	digitalWrite(pinCS, LOW);
	SPI.transfer(cmdByte);
	r = (unsigned long) SPI.transfer(0x0) << 16;
	r |= (unsigned long) SPI.transfer(0x0) << 8;
	r |= (unsigned long) SPI.transfer(0x0);
	digitalWrite(pinCS, HIGH);

	return r;
}

//write 24 bit data to register
void MCP3903::writeRegister(byte reg, unsigned long data)
{
	byte cmdByte = DEVICE_ADDR | reg <<1;
	
	byte b2 = (data & 0xff0000) >> 16;
	byte b1 = (data & 0x00ff00) >> 8;
	byte b0 = data & 0x0000ff;

	digitalWrite(pinCS, LOW);
	SPI.transfer(cmdByte);
	SPI.transfer(b2);
	SPI.transfer(b1);
	SPI.transfer(b0);
	digitalWrite(pinCS, HIGH);	
}

//read from ADC channel (0-5)
double MCP3903::readADC(byte channel)
{
	signed long r = readRegister(channel);

	if (r > 0x7FFFFF) r -= 0xFFFFFF;

	if (ScaleADCValue)
	{
	return (double) r / 0x800000 /3.0 * Gain[channel]*ReferenceVoltage;
	}
	else
	{
	return (double) r / 0x800000 /3.0*ReferenceVoltage ;
	}
}

//set channel gain
//channel (0-5)
//valid gain settings:
//GAIN_1
//GAIN_2
//GAIN_4
//GAIN_8
//GAIN_16
//GAIN_32
void MCP3903::setGain(byte channel, byte gain) 
{
	setGain(channel, gain, 0);
}

//overloaded setGain
//the boost parameter indicates the current boost setting for channel
void MCP3903::setGain(byte channel, byte gain, byte boost)
{
	unsigned long r = readRegister(REG_GAIN);

	
	Gain[channel]=CnvtGaincode2Gain(gain);
	
	byte idx = channel * 4;
	unsigned long chGain = 0;

	if (channel % 2 == 0) //0, 2, 4
	{
		chGain = (boost << 3) | gain;
	}
	else //1, 3, 5 
	{
		chGain = boost | (gain << 1);
	}
	
	r &= ~(0xf << idx);
	r |= chGain << idx;

	writeRegister(REG_GAIN, r);
}

byte MCP3903::CnvtGaincode2Gain(byte Gaincode)
{
		byte Gainfactor;
		switch (Gaincode){
		
		case 0: Gainfactor=1;
				break;
		case 1: Gainfactor=2;
				break;
		case 2: Gainfactor=4;
				break;
		case 3: Gainfactor=8;
				break;
		case 4: Gainfactor=16;
				break;
		case 5: Gainfactor=32;
				break;
		default: Gainfactor=0;
				break;
	}
	return Gainfactor;
}

void MCP3903::scaleReadADC( byte OnOff)
{
	ScaleADCValue=OnOff;
}

void MCP3903::setReferenceVoltage(float AdjRefVoltage)
{
	ReferenceVoltage=AdjRefVoltage;
}

byte MCP3903::getGain(byte channel)
{
	unsigned long r = readRegister(REG_GAIN);
	unsigned long chGain = 0;
	
	byte idx = channel * 4; //structure contains 4 bits
	r>>= idx;
	

	
	if (channel % 2 == 0) //0, 2, 4
	{
		chGain = (r & 0x07);
	}
	else //1, 3, 5 
	{
		chGain = (r & 0x0E)>>1;
	}

	return CnvtGaincode2Gain(chGain);
}

void MCP3903::setADCWidth(byte channel, byte width)//16 or 24 allowed
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	if (width==16)
	{
		r &= ~(1<<(15+channel));
	}
	else
	{
		r |= (1<<(15+channel));
	}
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDRA_Mode(byte mode)//0..3 allowed
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r &= ~(0x03<<6);
	r |= (mode<<6);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDRB_Mode(byte mode)//0..3 allowed
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(0x03<<8);
	r |=(mode<<8);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDRC_Mode(byte mode)//0..3 allowed
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(0x03<<10);
	r|=(mode<<10);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDR_Link_Mode(byte mode)//allowed 0=off, 1=on 
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(1<<12);
	r|=(mode<<12);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDR_HIZ(byte mode)//allowed 0=High impedancy, 1=High if not Ready
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(1<<13);
	r|=(mode<<13);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setDR_LTY(byte mode)//allowed 0=unsettled Data, 1=True no latency, data ready after 3 DRCLK-Periods
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(1<<14);
	r|=(mode<<14);
	writeRegister(REG_STATUS_COMM,r);
}

void MCP3903::setWMode(byte mode)//allowed 0=incremental addressing, 1=static addressing
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(1<<21);
	r|=(mode<<21);
	writeRegister(REG_STATUS_COMM,r);
}

byte MCP3903::getDataReady(byte channel)//allowed:0..5
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	
	return (byte) (r&(1<<channel));
}

void  MCP3903::setReadmode(byte mode)//allowed: 0=static, 1=groups, 2=types (default), 3=registermap
{
	unsigned long r = readRegister(REG_STATUS_COMM);
	r&=~(3<<22);
	r|=(mode<<22);
	writeRegister(REG_STATUS_COMM,r);
}

byte  MCP3903::getMod(byte channel)//allowed: 0..5
{
	byte offset;
	unsigned long r = readRegister(REG_MOD);
	offset=channel*4;
	r>>=offset;
	return (byte) r;
}

void  MCP3903::setPhase(byte chpair, byte phase)//allowed: chpair:01,23,45 phase:0..127
{
	unsigned long r = readRegister(REG_PHASE);
	switch (chpair){
		
		case 01: 
			r&=~(127);
			r|=(phase);
			break;
		case 23: 
			r&=~(127<<8);
			r|=(phase<<8);
			break;
		case 45: 
			r&=~(127<<16);
			r|=(phase<<16);
			break;
		default: break;
	}
	writeRegister(REG_PHASE,r);
}

void  MCP3903::setReset(byte channel,OnOff)//allowed: channel:0..5, OnOff=0 no reset/OnOff=1 reset
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(1<<18+channel);
	r|=(mode<<18+channel);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setShutdown(byte channel,OnOff)//allowed: channel:0..5, OnOff=0 no shutdown/OnOff=1 shutdown
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(1<<12+channel);
	r|=(mode<<12+channel);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setDither(byte channel,OnOff)//allowed: channel:0..5, OnOff=0 no dithering/OnOff=1 dithering
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(1<<6+channel);
	r|=(mode<<6+channel);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setOSR(byte mode)//allowed: OSR_32, OSR_64, OSR_128, OSR_256
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(3<<4);
	r|=(mode<<4);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setPrescale(byte mode)//allowed: 0..3
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(3<<2);
	r|=(mode<<2);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setExtRef(byte OnOff)//allowed: OnOff=0 internal Reference (default) /OnOff=1 external Reference requied
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(1<<1);
	r|=(mode<<1);
	writeRegister(REG_CONFIG,r);
}

void  MCP3903::setExtClk(byte OnOff)//allowed: OnOff=0 internal Oscillator (resonator required) /OnOff=1 external clock required
{
	unsigned long r = readRegister(REG_CONFIG);
	r&=~(1<<0);
	r|=(mode<<0);
	writeRegister(REG_CONFIG,r);
}

	
